;;; $DOOMDIR/config.el -*- lexical-binding: t; -*-

;; Place your private configuration here! Remember, you do not need to run 'doom
;; sync' after modifying this file!

;; Some functionality uses this to identify you, e.g. GPG configuration, email
;; clients, file temp+bindingslates and snippets. It is optional.
(setq user-full-name "Matt Nish-Lapidus"
      user-mail-address "matt@emenel.ca")

;; See 'C-h v doom-font' for documentation and more examples of what they
;; accept. For example:
(setq doom-font (font-spec :family "JetBrains Mono" :size 14))
(setq fancy-splash-image (concat doom-user-dir "splash.png"))
(setq doom-theme 'doom-monokai-pro)

(setq +doom-dashboard-pwd-policy nil)

;; This determines the style of line numbers in effect. If set to `nil', line
;; numbers are disabled. For relative line numbers, set this to `relative'.
(setq display-line-numbers-type 'relative)
(setq scroll-preserve-screen-position t)
(setq global-hl-line-modes nil)

(custom-set-faces!
  '(line-number :foreground "#424242"))

(setq locale-coding-system 'utf-8)
(set-terminal-coding-system 'utf-8)
(set-keyboard-coding-system 'utf-8)
(prefer-coding-system 'utf-8)

(setq org-directory "~/Documents/Obsidian/Primary")

(setq treesit-extra-load-path '("~/Source/_src/tree-sitter-module/dist/"))

(use-package! kbd-mode)

;; Modeline
;; - add current workspace name
;; - add major mode icon
(after! doom-modeline
  (setq doom-modeline-persp-name t
        doom-modeline-major-mode-icon t)
  (setq doom-modeline-total-line-number t))

(defun +default/insert-snippet ()
  (interactive)
  (cond ((modulep! :completion vertico)
         (call-interactively #'consult-yasnippet))
        (t (call-interactively #'yas-insert-snippet))))

;; set default frame size on launch
(setq default-frame-alist '( (left . 0) (top . 0) (height . 48) (width . 160)))

(defun mnl/frame-center ()
  "Center the current frame."
  (interactive)
  (let* ((dw (display-pixel-width))
         (dh (display-pixel-height))
         (f  (selected-frame))
         (fw (frame-pixel-width f))
         (fh (frame-pixel-height f))
         (x  (- (/ dw 2) (/ fw 2)))
         (y  (- (/ dh 2) (/ fh 2))))
    (message (format "dw %d dh %d fw %d fh %d x %d y %d" dw dh fw fh x y))
    (set-frame-position f x y)))

;; some global settings
(setq tab-always-indent t)              ; don't autocomplete with tab
(+global-word-wrap-mode +1)             ; default to soft word wrap
(delete-selection-mode 1)               ; delete stuff!
(setq delete-active-region t)           ; delete selected stuff!
(setq shell-file-name (executable-find "bash"))
(setq dired-vc-rename-file t) ; renaming and changing files in dired now integrates with magit

(after! vterm
  (setq-default vterm-shell (executable-find "fish"))
  (setq-default explicit-shell-file-name (executable-find "fish"))
  (setq vterm-kill-buffer-on-exit t)
  (after! meow
   (meow-vterm-enable)))

;; remove org agenda from the splash screen
(assoc-delete-all "Open org-agenda" +doom-dashboard-menu-sections)

;; reduce which-key waiting
(after! which-key
  (setq which-key-idle-delay 0.3))

;; some defaults for cursor size, scrolling, and buffer handling
(setq x-stretch-cursor t
      kill-buffer-delete-auto-save-files t
      scroll-conservatively 0)

;; nicer resizing
(cond
 ((string-equal system-type "darwin")
  (setq frame-resize-pixelwise  t
        window-resize-pixelwise t)))

;; tabs! i like them
;; (setq-default indent-tabs-mode t)
(setq-default tab-width 2)

(after! treemacs
  (setq treemacs-follow-mode t)
  (setq treemacs-project-follow-mode t))

(after! anzu
  (global-anzu-mode))

(global-so-long-mode 1)

(after! parinfer-rust-mode
  (setq parinfer-rust-auto-download t)
  (setq parinfer-rust-preferred-mode "smart"))

(setq dap-ui-variable-length 200)

;; smart tabs set modes
(after! smart-tabs-mode
  (smart-tabs-insinuate 'c 'javascript 'python))

;; Add consult-line searches to isearch history (both regular and regex) -- can now use meow 'n' to continue search as well as isearch
(advice-add #'consult-line :after
            (lambda (&rest _)
              (when consult--line-history
                (add-to-history
                 'regexp-search-ring
                 (car consult--line-history)
                 regexp-search-ring-max)
                (add-to-history
                 'search-ring
                 (car consult--line-history)
                 search-ring-max))))

;; bibliography file location
(after! citar
  (setq! citar-bibliography '("~/Zotero/lib.bib"))
  (setq! citar-notes-paths '("~/Documents/Obsidian/Primary/Sources"))
  (setq! citar-file-note-extensions '("md")))

;; svelte support
(add-to-list 'auto-mode-alist '("\\.svelte\\'" . web-mode))
(add-to-list 'auto-mode-alist '("\\.css\\'" . web-mode))
(setq web-mode-engines-alist
      '(("svelte" . "\\.svelte\\'")))

;; web-mode settings
(add-hook! web-mode (web-mode-toggle-current-element-highlight))
(after! web-mode
  (setq web-mode-markup-indent-offset 2)
  (setq web-mode-css-indent-offset 2)
  (setq web-mode-code-indent-offset 2))

(after! markdown-mode
  (setq markdown-list-indent-width 2)
  (setq markdown-enable-wiki-links t))

;; fix centaur tabs to show at the right times and add my muscle memory keybind
(after! centaur-tabs
  (setq centaur-tabs-set-bar 'right)
  (setq centaur-tabs-cycle-scope 'tabs)
  (map! :map centaur-tabs-mode-map
        "s-M-<right>" 'centaur-tabs-forward
        "s-M-<left>" 'centaur-tabs-backward))

;; use kitty for terminal-here
(setq terminal-here-mac-terminal-command '("kitty" "@" "launch" "--type=tab"))

;; set project paths and ignore buffers
(setq projectile-project-search-path '(("~/Source" . 2)))
(setq projectile-globally-ignored-buffers
      '("*scratch*"
        "*lsp-log*"))

;; ignoring buffers by their major mode
(setq projectile-globally-ignored-modes
      '("erc-mode"
        "help-mode"
        "completion-list-mode"
        "Buffer-menu-mode"
        "gnus-.*-mode"
        "occur-mode"))

;; mouse support for vertico buffers
(after! vertico
  (vertico-mouse-mode))

;; sly
(after! sly
  (setq sly-lisp-implementations
      '((sbcl ("sbcl" "--dynamic-space-size" "4Gb"))
        (ciel ("sbcl" "--core" "/Users/emenel/quicklisp/local-projects/CIEL/ciel-core" "--eval" "(in-package :ciel-user)" "--dynamic-space-size" "2600"))))
  (setq sly-default-lisp 'sbcl)
  (setq sly-complete-symbol-function 'sly-flex-completions))

;; cl-collider cl-patterns emacs helpers TODO: test this
(after! lisp-mode
  (defun cl-patterns-helpers-load ()
    (interactive)
    (sly-eval-async '(cl:namestring (asdf:system-source-directory (asdf:find-system 'cl-patterns)))
      (lambda (path)
        (load (concat path "res/emacs/cl-patterns-helpers") nil nil nil t)
        (load (concat path "res/emacs/cl-patterns-skeletons") nil nil nil t)))
    (define-key sly-mode-map (kbd "C-c p") 'cl-patterns-play-or-end-context-or-select-pdef)
    (define-key sly-mode-map (kbd "C-c P") 'cl-patterns-play-or-stop-context-or-select-pdef)
    (define-key sly-mode-map (kbd "C-c s") 'cl-patterns-stop-all)
    (define-key sly-doc-map (kbd "s") 'cl-patterns-supercollider-documentation)
    (add-hook! 'sly-connected-hook 'cl-patterns-helpers-load)))

(set-popup-rule! "*sly-description*" :side 'right :size '0.52)

(use-package! ajrepl
  :after janet-mode
  :config
  (add-hook 'a-janet-mode-hook
            #'ajrepl-interaction-mode))

(after! lsp-mode
  (setq lsp-signature-render-documentation nil)
  (add-to-list 'lsp-language-id-configuration '(a-janet-mode . "janet"))
  (lsp-register-client (make-lsp-client :new-connection (lsp-stdio-connection "janet-lsp")
                                        :activation-fn (lsp-activate-on "janet")
                                        :server-id 'janet-ls)))

(after! lisp-markup
  (add-hook! 'lisp-mode 'lisp-markup-minor-mode))
(add-to-list 'auto-mode-alist '("\\.lsx\\'" . lisp-mode))

;; adding supercollider path
(setq exec-path (append exec-path '("/Applications/SuperCollider.app/Contents/MacOS/")))

;; tidal cycles!
;; (setq tidal-boot-script-path "~/.cabal/share/aarch64-osx-ghc-9.4.2/tidal-1.9.3/BootTidal.hs")

;; add platformio to ino files
(add-to-list 'auto-mode-alist '("\\.ino\\'" . arduino-mode))
(add-to-list 'auto-mode-alist '("\\.ino\\'" . platformio-mode))

(after! gcode-mode
  (add-hook! 'gcode-mode-hook 'eldoc-mode))

;; (use-package! ready-player
;;   :config
;;   (ready-player-add-to-auto-mode-alist))

;; override aggressive key shortcuts for alt-l/r
(use-package! drag-stuff
  :defer t
  :init
  (map! "<M-up>"    #'drag-stuff-up
        "<M-down>"  #'drag-stuff-down
        "<M-left>"  #'left-word
        "<M-right>" #'right-word))

;; load additional config files
(load! "obsidian-config.el")
(load! "mu4e-config.el")

;; import all my customized keybindings based on doom evil mode, but with meow instead.
(load! "bindings.el")

(add-hook 'window-setup-hook #'mnl/frame-center)

(server-start)
